// Fusion display of Wikipedia user contributions history
// Copyright 2006, Marc Mongenet
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// See http://www.gnu.org/licenses/gpl.html

var fus = new Object();  // fus is Used as namespace.

fus.colors = [ "",
               "#FFB", "#FBF", "#BFF", "#DDF", "#DFD", "#FDD",
               "#EEA", "#EAE", "#AEE", "#CCE", "#CEC", "#ECC",
               "#DD9", "#D9D", "#9DD", "#BBD", "#BDB", "#DBB" ];
fus.color_idx = 0;

fus.makeContributorsList = function(contribLimit) {
	var list = document.createElement("ol");
	list.fus = new Object();
	list.id = "fus.contributors";
	list.fus.contribLimit = contribLimit;
	return list;
}

fus.addToContributorsList = function(list, contributor) {
	// UI components
	var li = document.createElement("li");
	li.fus = new Object();
	li.fus.contributor = contributor;
	li.fus.colorInput = document.createElement("input");
	li.fus.colorInput.type = "text";
	li.fus.colorInput.value = fus.colors[fus.color_idx++];

	// UI layout
	li.appendChild(document.createTextNode(contributor+", color: "));
	li.appendChild(li.fus.colorInput);
	li.style.backgroundColor = li.fus.colorInput.value;

	list.appendChild(li);
}

fus.getFromContributorsList = function(list, contributor) {
	for (var li = list.firstChild; li; li = li.nextSibling)
		if (li.fus.contributor == contributor)
			return li;
}

fus.monthNameToNum = function(m) {
	if (m == "janvier") return 1;
	if (m == "février") return 2;
	if (m == "mars") return 3;
	if (m == "avril") return 4;
	if (m == "mai") return 5;
	if (m == "juin") return 6;
	if (m == "juillet") return 7;
	if (m == "août") return 8;
	if (m == "septembre") return 9;
	if (m == "octobre") return 10;
	if (m == "novembre") return 11;
	if (m == "décembre") return 12;
	return 0;
}

// Return Date of contribution listed in this li element.
fus.parseContribTime = function(li) {
	var datetime_re = /(\d+) (\S+) (\d{4}) à (\d{2}):(\d{2})/;
	var datetime = li.firstChild.nodeValue.match(datetime_re);
	datetime[2] = fus.monthNameToNum(datetime[2]);
	return new Date(parseInt(datetime[3]),
		            datetime[2],
	                parseInt(datetime[1]),
	                parseInt(datetime[4], 10),
	                parseInt(datetime[5], 10));
}

fus.onFusionWith = function() {
	var contributor = document.getElementById("fus.contributor").value;
	var contributorsList = document.getElementById("fus.contributors");
	fus.addToContributorsList(contributorsList, contributor);

	var contributorLI = fus.getFromContributorsList(contributorsList, contributor);
	var contribLimit = contributorsList.fus.contribLimit;

	// Load fusionned page with synchronous XMLHttpRequest
	var http_request;
	if (window.XMLHttpRequest) { // Mozilla, Safari, ...
		http_request = new XMLHttpRequest();
		if (http_request.overrideMimeType)
			http_request.overrideMimeType('text/xml');
	} else if (window.ActiveXObject) { // IE
		try {
			http_request = new ActiveXObject("Msxml2.XMLHTTP");
		} catch (e) {
			try {
				http_request = new ActiveXObject("Microsoft.XMLHTTP");
			} catch (e) {}
		}
	}
	http_request.open('GET', "/w/index.php?title=Special:Contributions&target="+contributor+"&offset=0&limit="+contribLimit, false);
	http_request.send(null);

	// Get both contributions histories
	var ul1 = document.getElementById("bodyContent");
	ul1 = ul1.getElementsByTagName("ul")[0];
	var lis1 = ul1.getElementsByTagName("li");

	var lis2 = http_request.responseXML;
	lis2 = lis2.getElementById("bodyContent");
	lis2 = lis2.getElementsByTagName("ul")[0];
	lis2 = lis2.getElementsByTagName("li");

	// Fusion
	var i1 = 0;
	var li1 = lis1[i1];
	for (var i2 = 0; lis2[i2]; ++i2) {
		var li2 = lis2[i2].cloneNode(true);
		li2.appendChild(document.createTextNode(" -- "+contributor));
		li2.style.backgroundColor = contributorLI.fus.colorInput.value;
		var d2 = fus.parseContribTime(li2);

		while (li1) {
			var d1 = fus.parseContribTime(li1);
			if (d1 > d2)
				li1 = lis1[++i1];
			else
				break;
		}
		ul1.insertBefore(li2, li1);
	}
}

fus.parseContribUrlTarget = function(url) {
	var target = url.match(/&target=([^&]+)/);
	if (!target)
		target = url.match(/Special:Contributions\/(\S+)/);
	return target[1];
}

fus.parseContribUrlLimit = function(url) {
	var limit = url.match(/&limit=(\d+)/);
	if (limit)
		return parseInt(limit[1]);
	return 0;
}

fus.onContribPage = function(url) {
	// UI components
	var contributorsList = fus.makeContributorsList(fus.parseContribUrlLimit(url));
	fus.addToContributorsList(contributorsList, fus.parseContribUrlTarget(url));

	var input = document.createElement("input");
	input.type = "text";
	input.id = "fus.contributor";

	var button = document.createElement("button");
	button.onclick = fus.onFusionWith;

	// UI components layout
	button.appendChild(document.createTextNode("Fusionner"));
	var fusion = document.createTextNode("Fusionner avec les contributions de ");
	var bodyContent = document.getElementById("bodyContent");
	bodyContent.insertBefore(button, bodyContent.firstChild);
	bodyContent.insertBefore(input, bodyContent.firstChild);
	bodyContent.insertBefore(fusion, bodyContent.firstChild);
	bodyContent.insertBefore(contributorsList, bodyContent.firstChild);
}

fus.testForContribPage = function () {
//	try {
		var url = window.location.href;
		if (url.match(/\/Special:Contributions\/|title=Special:Contributions/))
			fus.onContribPage(url);
//	} catch (e) { /* Something went wrong. */ }
}

if (window.addEventListener) {
	window.addEventListener("load", fus.testForContribPage, false);
}
else if (window.attachEvent) {
	window.attachEvent("onload", fus.testForContribPage);
}