/*
	ce script (pour firefox) rassemble bon nombre de « gadgets » des préférences
	et permet une navigation plus rapide en centralisant les appels de scripts
*/

var Config = {

	NavigationTool : {

		addToolboxLinks : true,
		editFirstSection : true,
		addGotoTopLinks : true,
		changeModifyLinks : true,
		shortenRefBox : true,

	},

	NavigationToolOptions : {

		addToolboxLinks : {
			'Stats':'http://stats.grok.se/fr/',
			'Journal':'http://fr.wikipedia.org/w/index.php?title=Special:Journal&page=',
			'Sous pages':'http://fr.wikipedia.org/wiki/Special:Index/',
		},

		editFirstSection : '/w/index.php?title=articleName&action=edit&section=0',
		addGotoTopLinks : '⬆',
		changeModifyLinks : '✍',
		shortenRefBox : '300px',

	},

	EditingTool : {

		//edition options
		setAsMinorEdit : true,
		addPredefinedText : true,
		addRegexTool : true,
		addLessEditClutter : false,
	},

	EditingToolOptions : {
		setAsMinorEdit : 'wpMinoredit',
		addPredefinedText : {

			'main' : [
				'typo',
				'ortho',
				'wikif',
				'syntaxe',
				'interwiki',
				'homonymie',
				'ajout cat',
                                '+auteur/date',
                                '+refs',
			],

			'projets' : {
				'Projets':'',
				'----':'',
				'Liens rouges':'[[Projet:Restauration_lien_rouge]]',
				'Homonymie':'',
			},

			'bandeaux' : {
				'Bandeaux':'',
				'----':'',
				'Ébauche':'{{m|Ébauche}}',
				'Portail':'{{m|Portail}}',
				'Copyvio':	'{{m|Copyvio}}',
				'À recycler':'{{m|À recycler}}',
			},

			'blanchiment' : {
				'Blanchiment':'',
				'----':'',
				'Bac à sable':'blanchi (bac à sable)',
				'Canular':'blanchi (canular)',
				'HC':'blanchi ([[WP:HC]])',
			}
		}
	}
}

var Helpers = {

	createNewNode : function(type, attrDict, parentNode) {
		var node = document.createElement(type);
		for (key in attrDict) node[key] = attrDict[key];
		return parentNode ? parentNode.appendChild(node) : node;
	},

	getElement : function(str, parent) {
		parent = parent || document;
		return document.getElementById(str) || parent.getElementsByClassName(str)[0] || parent.getElementsByTagName(str)[0];
	},

	getElements : function(str, parent) {
		parent = parent || document;
		return parent.getElementsByClassName(str).length==0 ? parent.getElementsByTagName(str) : parent.getElementsByClassName(str);
	},

	getSibling : function(elm, dir) {
		while (elm[dir].data == '\n')
			elm = elm[dir];
		return elm[dir];
	},

	getNextSibling : function(elm) {
		return Helpers.getSibling(elm,'nextSibling');
	},

	getPreviousSibling : function(elm) {
		return Helpers.getSibling(elm,'previousSibling');
	},

	appendCustom : function ( elm, arr ) {
		for (var i=0;i<arr.length;i++) {
			if (typeof(arr[i]) == "string")
				arr[i] = document.createTextNode(arr[i]);
			elm.appendChild(arr[i]);
		}
	}
}

// shortcuts for Helpers
$new = Helpers.createNewNode;
$get = Helpers.getElement;
$$get = Helpers.getElements;
$next = Helpers.getNextSibling;
$previous = Helpers.getPreviousSibling;
$append = Helpers.appendCustom;

var NavigationTool = {

	menu : null,
	articleName : wgTitle,

	init : function() {

		var _this = 'NavigationTool';
		for(option in Config[_this])
			if (Config[_this][option])
				NavigationTool[option](Config[_this+'Options'][option]);

	},

	addToolboxLinks : function(dict) {

		NavigationTool.menu = $get('ul', $get('p-tb'));

		for (key in dict) {
			if (key=='Stats') {
				var url = dict['Stats']+NavigationTool.generateDateLink()+'/';
				NavigationTool.addToolboxLink('Stats',url);
			} else
				NavigationTool.addToolboxLink(key,dict[key]);
		}
	},

	generateDateLink : function() {
		var date = new Date();
		date.setMonth(date.getMonth()-1);
		return date.getFullYear()+''+(date.getMonth()+1<10?'0'+(date.getMonth()+1):date.getMonth()+1);
	},

	addToolboxLink : function(name, loc) {
		var link = $new('li',{},NavigationTool.menu);
		$new('a',{innerHTML:name,href:loc+NavigationTool.articleName},link);
	},

	editFirstSection : function(text) {
		var head = $get('h1');
		var link = text.split('articleName').join(NavigationTool.articleName);
		var linkContent = '[<a href="'+link+'">modifier</a>]';
		$new('span',{innerHTML:linkContent,className:'editsection'},head);
	},

	changeModifyLinks : function(text) {
		var links = $$get('editsection');
		for (var i=0;i<links.length;i++) {
			var link = $get('a',links[i]).href;
			links[i].innerHTML = '';
			$new('a',{href:link,innerHTML:text},links[i]);
		}
	},

	addGotoTopLinks : function(text) {

		var bodyContent = $get('bodyContent');
		var titles = [];

		for (var i=2;i<=6;i++)
			titles = titles.concat(NavigationTool.extractTags('h'+i, bodyContent));

		for(var i=0;i<titles.length;i++)
			$new('a',{innerHTML:text,href:'#top'},titles[i]);

	},

	extractTags : function(tag, parent) {
		var collection = $$get(tag, parent);
		var tempArr = [];
		for(var i=0;i<collection.length;i++)
			tempArr.push(collection[i]);
		return tempArr;
	},

	shortenRefBox : function(height) {
		if (NavigationTool.articleName.match(/(Wikipédia|Discussion)/)) return;
		var refs = $$get('references');
		if (refs.length>0) {
			for (var i=0;i<refs.length;i++) {
                                var refBlock = refs[i].parentNode;
                                if (refBlock.id == "bodyContent") continue;
				refBlock.style.maxHeight = height;
				refBlock.style.overflowY = 'auto';
				refBlock.style.overflowX = 'hidden';
				if (refBlock.parentNode.style.MozColumnCount != '1') refBlock.parentNode.style.MozColumnCount = '1';
                                refBlock.style.MozColumnCount = '2';
			}
		}
	},

}

var RegexTool = {
	template : '<input type="text" id="regexToolFind" value=""/><input type="text" id="regexToolReplace" value=""/><input type="checkbox" id="regexToolDoReplace" /><label for="regexToolDoReplace">Replace</label>',
	textarea : null,
	results : null,
	msgType : null,
	flag : null,
	find : null,
	replace : null,
	matchValues : null,

	init : function () {
		RegexTool.textarea = $get('textarea');
		var link = $new('a',{innerHTML:"[Regex]",id:"regexToolLink",href:"#",onclick:RegexTool.showRegexTool});
		$get('bodyContent').insertBefore(link, $get('editform'));
		$new('style',{innerHTML:'#bodyContent, #regexTool {position:relative;} #RegexToolMatchValues, #regexToolResults, #regexToolLink, #regexToolButton {position:absolute;right:0;top:0px;} #regexToolButton {cursor:pointer;} #regexToolResults {right:0;top:-28px;} #regexToolDoReplace {margin:3px 0 0 5px;} #RegexToolMatchValues {display:none;border:1px solid black;background:#FFFFEF;top:30px !important;}'},$get('head'));
	},

	showRegexTool : function () {
		var bodyContent = $get('bodyContent');
		var regexMain = $new('div',{id:'regexTool',innerHTML:RegexTool.template});
		$new('div',{id:'regexToolButton',onclick:RegexTool.parseFields,innerHTML:'Go!'},regexMain);
		RegexTool.results = $new('div',{id:'regexToolResults'},regexMain);
		bodyContent.insertBefore(regexMain, $get('editform'));
		bodyContent.removeChild(this);
		return false;
	},

	arrayCount : function(arr) {
		arr = arr.sort();
		var tempKey = '';
		var tempCount = '';
		var dict = {};
		for (i in arr) {
			if (arr[i]==tempKey) dict[arr[i]]++
			else {dict[arr[i]] = 1;	tempKey=arr[i];};
		}
		return dict;
	},

	makeMatchList : function(dict) {
		var str = '';
		for (key in dict) str+=key+': '+dict[key]+'<br />';
		return str;
	},

	showInfo : function(type, addInfo) {
		messages = [
			'Empty Field',
			'Found None',
			'Found: ',
			'Replaced: '
		]
		addInfo = addInfo || '';
		RegexTool.results.innerHTML = messages[type]+addInfo;
	},

	parseFields : function() {
		var find = $get('regexToolFind').value;
		RegexTool.flag = $get('regexToolDoReplace').checked;
		RegexTool.replace = $get('regexToolReplace').value;
		if (find=='' || (RegexTool.flag && RegexTool.replace==''))
			RegexTool.showInfo(0)
		else {
			RegexTool.find=new RegExp(find,'g');
			RegexTool.parseContent();
		}
	},

	parseContent : function () {

		var text = RegexTool.textarea.value;

		try {
			var str = text.match(RegexTool.find);
			var matchCounter = str.length;
		} catch (e) {
			RegexTool.showInfo(1);
			return;
		}

		if (RegexTool.flag) {
			msgType = 3
			RegexTool.textarea.value = text.replace(RegexTool.find, RegexTool.replace);
		} else msgType = 2;

		// display number of matches and a popup with different ones
		RegexTool.showInfo(msgType,matchCounter);
		RegexTool.displayMatches(str);
	},

	displayMatches : function (str) {
		var content = '<b>Matches:</b><br />'+RegexTool.makeMatchList(RegexTool.arrayCount(str));
		RegexTool.matchValues = $new('div',{id:'RegexToolMatchValues',innerHTML:content},RegexTool.results);
		var link = $new('a',{id:'RegexToolMatchLink',innerHTML:'<sup>[info]</sup>',href:'#'},RegexTool.results);
		link.onmouseover = function() {RegexTool.matchValues.style.display = 'block';}
		link.onmouseout = function() {RegexTool.matchValues.style.display = 'none';}
	}
}


var EditingTool = {

	wpSummary : null,

	init : function() {
		var _this = 'EditingTool';
		for(option in Config[_this])
			if (Config[_this][option])
				EditingTool[option](Config[_this+'Options'][option]);
	},

	setAsMinorEdit : function(id) {
		$get(id).checked="checked";
	},

	addPredefinedText : function() {

		// get config for keywords
		var keyDict = Config.EditingToolOptions.addPredefinedText;
		EditingTool.wpSummary = $get('wpSummary');

		// create div container
		var container = $get('editOptions');
		var firstSibling = $get('wpSummaryLabel');
		var keyDiv = $new('div',{id:'EditingToolMainDiv'});
		container.insertBefore(keyDiv,firstSibling);

		for (key in keyDict) {
			if (key=='main')
				for (key2 in keyDict['main'])
					$new('a',{href:'#',onclick:EditingTool.insertWord,innerHTML:keyDict['main'][key2]},keyDiv);
			else
				EditingTool.createSelectBox(keyDict[key],keyDiv);
		}
	},

	insertWord : function() {
		var text = this.tagName == 'A' ? this.innerHTML : this.value;
		EditingTool.wpSummary.value += text+' ';
		return false;
	},

	createSelectBox : function(dict, parent) {
		var select = $new('select',{onchange:EditingTool.insertWord},parent)
		for (key in dict)
			$new('option',{value:dict[key],innerHTML:key},select);
	},

	addRegexTool : function() {
		RegexTool.init();
	},

	addLessEditClutter : function() {
		LessEditClutter.init();
	}
}


window.addEventListener('DOMContentLoaded',loadFuncs,true);
function loadFuncs() {
	if (wgAction=="view") NavigationTool.init();
	if (wgAction=="edit" || wgAction=="submit") EditingTool.init();
}