// <nowiki>

function BoutonsHelpers($) {

    //--------------------------------------------------------------------------
    //  Bricolage pour avoir un dialog en position fixed :\
    //--------------------------------------------------------------------------
    function fixedDialog(jElement, options) {

        var immutableOptions = {
            autoOpen: false,
            dragStop: function (event, ui) {
                var $this = $(this);
                $this.dialog('option', 'position', [ui.position.left, ui.position.top]);
                $this.data('savePosition', [ui.position.left, ui.position.top]);
            },
            resizeStop: function (event, ui) {
                var $this = $(this);
                var $doc = $(document);
                var scrollLeft = $doc.scrollLeft();
                var scrollTop = $doc.scrollTop();

                // l'ordre de ces 2 lignes est important :
                $this.dialog('option', 'position', [ui.position.left - scrollLeft * 2, ui.position.top - scrollTop * 2]);
                $this.parent().css('position', 'fixed');

                $this.data('savePosition', [ui.position.left - scrollLeft, ui.position.top - scrollTop]);
            },
            close: function () {
                var $this = $(this);
                var savePosition = $this.data('savePosition');
                if (savePosition) {
                    $this.dialog('option', 'position', savePosition);
                }
            }
        };

        var dialogOptions = options ? $.extend({}, options) : {};

        var clientProfile = $.client.profile();
        if (clientProfile.name === 'msie' && clientProfile.versionNumber <= 6) {
            // fallback IE 6 : dialog normal
            jElement.dialog(dialogOptions);
        } else {
            $.extend(dialogOptions, immutableOptions);
            jElement.dialog(dialogOptions);
            // l'ordre de ces 2 lignes est important :
            jElement.parent().css('position', 'fixed');
            jElement.dialog('open');
        }
    };


    //--------------------------------------------------------------------------
    //  Fonctions utilitaires
    //--------------------------------------------------------------------------

    function toggleDialog(jDialog) {
        if (jDialog.dialog('isOpen')) {
            jDialog.dialog('close');
        } else {
            jDialog.dialog('open');
        }
    }

    // cf. http://help.dottoro.com/ljhxklln.php
    function onInputChange(domInput, callback) {
        if (domInput.addEventListener) {
            domInput.addEventListener('input', callback, false); // Firefox, etc.
        } else if (domInput.attachEvent) {
            domInput.attachEvent('onpropertychange', callback); // IE
        }
    }


    //--------------------------------------------------------------------------
    //  Code des boutons
    //--------------------------------------------------------------------------

    function boutonFactory(parametres) {
        var jDialog, domEntree, domSortie, initStarted, dialogReady;
        function updateResultat() {
            parametres.callback(domEntree, domSortie);
        }
        function initDialog() {
            var jEntree = $('<input type="text" style="width:98%" />');
            var jSortie = $('<input type="text" style="width:98%" />');
            domEntree = jEntree[0];
            domSortie = jSortie[0];
            onInputChange(domEntree, updateResultat);
            jDialog = $('<div />').append(jEntree, '<br /><br />', jSortie);
            fixedDialog(jDialog, {
                title: parametres.titreDialog,
                width: parametres.largeur
            });
            dialogReady = true;
        }
        function clickHandler() {
            if (!dialogReady) {
                if (!initStarted) {
                    initStarted = true;
                    mw.loader.using(['jquery.ui.dialog'], initDialog); // rappel : le chargement est asynchrone
                }
                return;
            }
            toggleDialog(jDialog);
        }
        var bouton = $('<button type="button">' + parametres.titreBouton + '</button>');
        bouton.click(clickHandler);
        $('#editform').find('div.editButtons').append(bouton);
    }

    // TODO : support URL https et interwikis
    boutonFactory({
        titreBouton: 'fullurl',
        titreDialog: 'Conversion URL brute vers {{fullurl:}}',
        largeur: 340,
        callback: function (domEntree, domSortie) {
            var valeurEntree = $.trim(domEntree.value);
            var titre = mw.util.getParamValue('title', valeurEntree).replace(/_/g, ' ');
            var paramsGet = valeurEntree.replace('http://fr.wikipedia.org/w/index.php', '');
            var paramsGetSansTitle = paramsGet.replace(/\?title=[^&]*&?/, '').replace(/&title=[^&]*/, '');
            domSortie.value = '{{fullurl:' + titre + '|' + decodeURIComponent(paramsGetSansTitle) + '}}';
        }
    });

    boutonFactory({
        titreBouton: 'lien modèle',
        titreDialog: 'Lien modèle pour résumé de modification',
        largeur: 340,
        callback: function (domEntree, domSortie) {
            function ucFirst(string) {
                return string.charAt(0).toUpperCase() + string.slice(1);
            }
            var valeurEntree = $.trim(domEntree.value);
            domSortie.value = '[[Modèle:' + ucFirst(valeurEntree) + '|{{' + valeurEntree + '}}]]';
        }
    });
}

if (mw.config.get('wgAction') === 'edit' || mw.config.get('wgAction') === 'submit') {
    $(document).ready(BoutonsHelpers);
}