String.prototype.htmlize = function() {
  var chars = new Array('&','<','>','"');
  var entities = new Array('amp','lt','gt','quot');
  var string = this;
  for (var i=0; i<chars.length; i++) {
    var regex = new RegExp(chars[i], "g");
    string = string.replace(regex, '&' + entities[i] + ';');
  }
  return string;
}

var lsdFlagValue = {
  NO: 0,
  IGN: 1,
  GREY: 2,
  YES: 3
};

var lsdFlags = new Object();
var lsdNamespace = "all";
var lsdInvert = false;
var lsdGroups = ' ' + wgUserGroups.join(' ') + ' ';
var lsdAutpatrolled = lsdGroups.indexOf(' autopatrolled ') != -1;
var lsdNumPages = -1;

lsdFlags['minor'] = lsdFlagValue.YES;
lsdFlags['bot'] = lsdFlagValue.YES;
lsdFlags['archivebot'] = lsdFlagValue.YES;
lsdFlags['iwbot'] = lsdFlagValue.YES;
lsdFlags['user'] = lsdFlagValue.YES;
lsdFlags['patrolled'] = lsdFlagValue.YES;
lsdFlags['revert'] = lsdFlagValue.YES;
lsdFlags['maintenance'] = lsdFlagValue.YES;
lsdFlags['eval'] = lsdFlagValue.YES;
lsdFlags['self'] = lsdFlagValue.YES;

function lsdHasClass(node, cls) {
  var classes = ' ' + node.className + ' ';
  return classes.indexOf(' ' + cls + ' ') != -1;
}

function lsdElementsByClass(node, class, tag) {
  var classEls = new Array();
  var els = node.getElementsByTagName(tag ? tag : '*');
  var pattern = new RegExp('(^|\\s)' + class + '(\\s|$)');
  for (var i = 0; i < els.length; i++) {
    if (pattern.test(els[i].className))
      classEls.push(els[i]);
  }
  return classEls;
}

function lsdGetBC() {
  return document.getElementById("bodyContent");
}

function lsdMakeStateLinks(param) {
  var states = ['N', 'n', 'o', 'O'];
  var statesTitle = ["Ne pas afficher, masque les modifications précédentes",
    "Ne pas afficher, sans masquer les modifications précédentes (ne marche qu'avec la liste de suivi étendue)",
    "Afficher en gris", "Afficher"];
  var html = '';
  for (var i = 0; i < states.length; i++) {
    if (i > 0) html += '&nbsp;|&nbsp;';
    if (lsdFlags[param] == i)
      html += '<b>' + states[i] + '</b>';
    else
      html += '<a href="javascript:;" onclick="lsdSetFlags(\'' + param + '\',' + i + ');" title="' + statesTitle[i] + '">' + states[i] + '</a>';
  }
  return '(' + html + ')';
}

function lsdMakeDaysLink(count, text) {
  return '<a href="' + wgServer + wgScriptPath + '/index.php?title=Special:Watchlist&days=' + count + '">' + text + '</a>';
}

function lsdCombineFlags(f1, f2) {
  var flagPrio = [2, 3, 1, 0];
  return (flagPrio[f1] > flagPrio[f2]) ? f1 : f2;
}

function lsdSetNamespace() {
  lsdNamespace = document.getElementById('lsdNamespace').value;
  lsdInvert = document.getElementById('lsdInvert').checked;
  lsdSort(lsdGetBC(), true);
}

function lsdSetFlags(name, value) {
  lsdFlags[name] = value;
  lsdSort(lsdGetBC(), false);
}

function lsdSaveCookie() {
  var s = 'ns-' + lsdNamespace;
  if (lsdInvert) s += '-nsi-1';
  for (f in lsdFlags) {
    s += '-' + f + '-' + lsdFlags[f];
  }
  var expDate = new Date()
  expDate.setTime(expDate.getTime() + (30 * 86400 * 1000));
  document.cookie = "lsdFlags=" + encodeURIComponent(s) + "; expires=" + expDate.toGMTString() + "; path=/;";
}

function lsdReadCookie() {
  var s = "; " + document.cookie + ";";
  var i = s.indexOf("; lsdFlags=") + 11;
  if (i < 11) return;
  s = decodeURIComponent(s.substr(i, s.indexOf(";", i) - i));
  if (s == "") return;
  fields = s.split('-');
  lsdInvert = false;
  for (i = 0; i < fields.length; i += 2) {
    if (fields[i] == 'ns')
      lsdNamespace = fields[i + 1];
    else if (fields[i] == 'nsi')
      lsdInvert = true;
    else {
      var x = parseInt(fields[i + 1]);
      if (x >= lsdFlagValue.NO && x <= lsdFlagValue.YES)
        lsdFlags[fields[i]] = x;
    }
  }
}

function lsdSort(bc, noFieldUpdate) {
  var undoTable = new Array();
  var regexpRevId = new RegExp(/&diff=([0-9]+)/);
  var regexpRv1 = new RegExp(/^Annulation des modifications ([0-9]+) /);
  var regexpRv2 = new RegExp(/Révocation des modifications de (.*) \(retour à la dernière version de (.*)\)($| ;)/);
  var regexpRv3 = new RegExp(/Révocation des modifications de (.*) \(discussion\) vers la dernière version de (.*)$/);
  var regexpRv4 = new RegExp(/bot : révocation de (.*) \(modification suspecte.*retour à la version [0-9]+ de (.*)$/);
  var regexpEval1 = new RegExp(/(^|\b)([Rr][ée]?)?[ÉéEe]val(uation|ué)?s?($|\b)/);
  var regexpEval2 = new RegExp(/(^|\b)[Ww]i?[Kk]i?projet($|\b)/);
  var regexpEval3 = new RegExp(/^(avancement|article promu|article déchu|passage en)\b/);
  var regexpMaintenance = new RegExp(/(^|\b)(WPCleaner|P:CS|[Cc]orrection syntaxique|using AWB)(\b|$)/);
  var regexpIP = /^\d{1,3}\.\d{1,3}.\d{1,3}.\d{1,3}$/;

  var i, match, articles = new Object(), clsPrefix = " watchlist-";
  var cfgFrame = bc.getElementsByTagName("fieldset")[0];
  if (cfgFrame && !noFieldUpdate) {
    if (lsdNumPages == -1) {
      match = new RegExp(/Votre liste de suivi référence ([0-9]+) pages/).exec(cfgFrame.textContent);
      lsdNumPages = match ? match[1] : -2;
    }
    var cfgHTML;
    cfgHTML = '<legend>Options de la liste de suivi</legend>';
    if (lsdNumPages != -2) {
      cfgHTML += '<p>Votre liste de suivi référence ' + lsdNumPages + ' pages, sans compter les pages de discussion.</p><hr />';
    }
    cfgHTML += '<p>Nombre de jours : '
      + lsdMakeDaysLink(1, '1 jour') + ', ' + lsdMakeDaysLink(3, '3 jours') + ', '
      + lsdMakeDaysLink(7, '7 jours') + ', ' + lsdMakeDaysLink(0, 'tout') + '.<br />'
      + 'Type : mineures ' + lsdMakeStateLinks('minor')
      + ', bots ' + lsdMakeStateLinks('bot')
      + ', bots d\'archivage ' + lsdMakeStateLinks('archivebot')
      + ', bots interwikis ' + lsdMakeStateLinks('iwbot')
      + ', utilisateurs inscrits ' + lsdMakeStateLinks('user')
      + (lsdAutpatrolled ? ', relues ' + lsdMakeStateLinks('patrolled') : '')
      + ', maintenance ' + lsdMakeStateLinks('maintenance')
      + ', évaluations ' + lsdMakeStateLinks('eval')
      + ', révocations ' + lsdMakeStateLinks('revert')
      + ', mes modifications ' + lsdMakeStateLinks('self') + '.</p><hr />'
      + '<p><label for="lsdNamespace">Espaces de noms&nbsp;:</label> '
      + '<select id="lsdNamespace" name="lsdNamespace" class="namespaceselector" onchange="lsdSetNamespace();"><option value="all">Tous</option>';
    for (var nsIndex in wgFormattedNamespaces) {
      if (nsIndex < 0) continue;
      cfgHTML += '<option';
      if (nsIndex == lsdNamespace) cfgHTML += ' selected="selected"';
      cfgHTML += ' value="' + nsIndex + '">' + (nsIndex > 0 ? wgFormattedNamespaces[nsIndex] : "Principal") + '</option>';
    }
    cfgHTML += '</select> <input onclick="lsdSetNamespace()" name="lsdInvert" value="1" id="lsdInvert" type="checkbox"';
    if (lsdInvert) cfgHTML += ' checked="checked"';
    cfgHTML += '>&nbsp;<label for="lsdInvert">Inverser la sélection</label> <input value="Enregistrer" title="Enregistrer les options dans un cookie" type="button" onclick="lsdSaveCookie();"></p>';
    cfgFrame.innerHTML = cfgHTML;
  }
  liNodes = bc.getElementsByTagName("li");
  for (i = 0; i < liNodes.length; i++) {
    var cls = " " + liNodes[i].className + " ";
    var k = cls.indexOf(clsPrefix);
    if (k == -1) continue;
    k += clsPrefix.length;
    var article = cls.substr(k, cls.indexOf(" ", k) - k);
    var namespace = parseInt(article);
    var goodNamespace = lsdNamespace == "all" || (lsdInvert != (lsdNamespace == namespace));

    var user = '';
    var userNode = lsdElementsByClass(liNodes[i], "mw-userlink", "a")[0];
    if (userNode) user = userNode.textContent;
    var notIP = !regexpIP.test(user);
    var self = user == wgUserName;

    var revid = '', links = liNodes[i].getElementsByTagName("a");
    for (var j = 0; j < links.length; j++) {
      match = regexpRevId.exec(links[j].href);
      if (match) {
        revid = match[1];
        break;
      }
    }

    // Le test sur revid sert à ne pas masquer des entrées du journal même s'il y en a plusieurs
    var flag = ((!articles[article] || revid == '') && goodNamespace) ? lsdFlagValue.YES : lsdFlagValue.IGN;
    var bot = false, iwbot = false, archiveBot = false, patrolled = true;
    var revert = false, reverted = 0, eval = false, maintenance = false;

    var abbr = liNodes[i].getElementsByTagName("abbr");
    for (var j = 0; j < abbr.length; j++) {
      if (lsdHasClass(abbr[j], "minor"))
        flag = lsdCombineFlags(flag, lsdFlags["minor"]);
      else if (lsdHasClass(abbr[j], "bot"))
        bot = true;
      else if (lsdHasClass(abbr[j], "unpatrolled"))
        patrolled = false;
    }

    var comment = lsdElementsByClass(liNodes[i], "comment", "span")[0];

    if (undoTable[article]) {
      if (undoTable[article] == "user=" + user) {
        reverted = 2;
      } else if (undoTable[article] == "user1=" + user) {
        reverted = 2;
        undoTable[article] = false;
      } else if (revid && undoTable[article] == "id=" + revid) {
        reverted = 2;
        undoTable[article] = false;
      } else if (undoTable[article].substr(0, 5) != "user=") {
        reverted = 1;
        undoTable[article] = false;
      }
    }

    if (comment) {
      var comTxt = comment.textContent;
      comTxt = comTxt.substr(1, comTxt.length - 2);
      if (bot) {
        archiveBot = (user == 'BotdeSki' && comTxt.substr(0, 13) == 'Archivage de ')
          || comTxt == 'Archivage des anciennes requêtes refusées'
          || comTxt == 'Archivage des anciennes requêtes traitées'
          || comTxt == 'Bot: archivage des requêtes aux bots'
          || comTxt == 'Bot: archivage des sections non modifiées depuis plus de 7 jours'
          || comTxt == 'Classement des demandes de protection de pages'
          || comTxt == 'Classement des demandes de renommage'
          || comTxt == 'Classement des demandes de restauration de pages'
          || comTxt == 'Classement des requêtes aux administrateurs'
          || comTxt == 'Suppression des anciennes requêtes acceptées';
        iwbot = namespace == 0 && user != 'Geobot' && lsdElementsByClass(comment, "extiw", "a").length > 0;
      }
      if (patrolled && reverted == 0) {
        match = regexpRv1.exec(comTxt);
        if (match) {
          undoTable[article] = "id=" + match[1];
          revert = true;
        }
        else {
          match = regexpRv2.exec(comTxt);
          if (!match) match = regexpRv3.exec(comTxt);
          if (!match) match = regexpRv4.exec(comTxt);
          if (match && match[1] != '') {
            if (match[1] != match[2]) {
              undoTable[article] = "user=" + match[1];
              revert = true;
            } else {
              undoTable[article] = "user1=" + match[1];
              revert = true;
            }
          }
        }
      }
      if (namespace == 1 && lsdElementsByClass(comment, "autocomment", "span").length == 0
          && (regexpEval1.test(comTxt) || regexpEval2.test(comTxt) || regexpEval3.test(comTxt))) {
        eval = true;
      }
      if (namespace == 0 && patrolled && regexpMaintenance.test(comTxt)) {
        maintenance = true;
      }
    }

    if (bot)        flag = lsdCombineFlags(flag, lsdFlags["bot"]);
    if (archiveBot) flag = lsdCombineFlags(flag, lsdFlags["archivebot"]);
    if (iwbot)      flag = lsdCombineFlags(flag, lsdFlags["iwbot"]);
    if (patrolled)  flag = lsdCombineFlags(flag, lsdFlags["patrolled"]);
    if (eval)       flag = lsdCombineFlags(flag, lsdFlags["eval"]);
    if (revert || reverted == 2) flag = lsdCombineFlags(flag, lsdFlags["revert"]);
    if (notIP)      flag = lsdCombineFlags(flag, lsdFlags["user"]);
    if (self)       flag = lsdCombineFlags(flag, lsdFlags["self"]);
    if (maintenance) flag = lsdCombineFlags(flag, lsdFlags["maintenance"]);

    liNodes[i].style.display = (flag == lsdFlagValue.NO || flag == lsdFlagValue.IGN) ? "none" : "";
      //liNodes[i].style.display = "";
    liNodes[i].style.fontSize = (flag == lsdFlagValue.GREY) ? "70%" : "";
    var color = (flag == lsdFlagValue.GREY) ? "#a88" : "";
      /*if (flag == lsdFlagValue.NO || flag == lsdFlagValue.IGN) {
        if (articles[article] && false)
          liNodes[i].style.display = "none";
        else {
          liNodes[i].style.fontSize = "70%";
          color = "#aaf";
        }
      }*/
    liNodes[i].style.color = color;
    var lineNodes = liNodes[i].getElementsByTagName('*');
    for (var j = 0; j < lineNodes.length; j++) {
      lineNodes[j].style.color = color;
    }
    if (flag != lsdFlagValue.IGN)
      articles[article] = true;
  }
}

if (wgCanonicalSpecialPageName == "Watchlist" && document.URL.indexOf("edit") == -1 && document.URL.indexOf("raw") == -1) {
  addOnloadHook(function () {
    lsdReadCookie();
    lsdSort(lsdGetBC(), false);
  });
}