  /******************************************************************************/
  /*                                                                            */
  /*                              Remerciements                                 */
  /*                                                                            */
  /******************************************************************************/
  
  /*
  *  Peu de choses dans cette page ont été écrites par moi-même.
  *  Je me suis très souvent inspiré des monobooks de Dake, Hégésippe Cormier,
  *  Korrigan, Meanos, Sam Hocevar et Solensean en y apportant
  *  quelquefois des modifications. Je tiens à les remercier ici.
  */



  /******************************************************************************/
  /*                                                                            */
  /*                               addLoadEvent                                 */
  /*                                                                            */
  /******************************************************************************/
  
  function addLoadEvent(func)
  {
      if (window.addEventListener)
      {
          window.addEventListener("load", func, false);
      }
      else if (window.attachEvent)
      {
          window.attachEvent("onload", func);
      }
  }



  /******************************************************************************/
  /*                                                                            */
  /*                                  bissextile                                */
  /*                                                                            */
  /******************************************************************************/
  
  /*	Détermine si une année est bissextile
   *
   *		annee (entier)
   *
   *		Renvoie "true" ssi 'annee' est bissextile
   *
   */
  
  function bissextile(annee)
  {
      if ( (annee % 4) != 0)
          return false;
      else
          return ( ((annee % 400) == 0) || ((annee % 100) != 0) );
  }



  /******************************************************************************/
  /*                                                                            */
  /*                                 semaine                                    */
  /*                                                                            */
  /******************************************************************************/
  
  /*	Détermine le numéro de la semaine pour une date donnée
  *
  *		uneDate (Date) : date pour laquelle on veut connaître la semaine
  *
  *		renvoie le numéro de la semaine(entier compris entre 1 et 53)
  *
  */
  
  function semaine(uneDate)
  {
      var nbJours = new Array(31,28,31,30,31,30,31,31,30,31,30);
      
      var anneeBissextile = bissextile(uneDate.getFullYear());
      if (anneeBissextile)
      {
          ++nbJours[2]; // plus 1 jour au mois de février !
      } 
      
      var cumul = new Array(12);
      
      cumul[0] = 0;
      for(mois = 0; mois < 11; ++mois)
      {
          cumul[mois + 1] = cumul[mois] + nbJours[mois];
      }
      
      jourSemaine = (uneDate.getDay() + 6) % 7;
      
      var lundi = uneDate.getDate() - jourSemaine + cumul[uneDate.getMonth()];
      
      var premierLundiJanvier = ((lundi + 7) % 7);
      if (premierLundiJanvier == 0)
      {
          premierLundiJanvier = 7;
      }
      
      var offset = (premierLundiJanvier > 4) ? 1 : 0;
      
      var S = Math.ceil(lundi / 7) + offset;
      
      // traitement des cas particuliers : débuts et fin d'année
      
      if (S == 0 )
      {
          // calcul sur l'année précédente
          var saintSylvestre = new Date(uneDate.getFullYear() - 1, 11, 31);
          var saintSylvestreGetDay = saintSylvestre.getDay(); 
          if( anneeBissextile )
          {
              if ( (saintSylvestreGetDay != 4) && (saintSylvestreGetDay != 5) )
              {
                  S = 52;
              }
              else
              {
                  S = 53;
              }
          }
          else
          {
              if ( (saintSylvestreGetDay != 4) )
              {
                  S = 52;
              }
              else
              {
                  S = 53;
              }
          }
      }
      else if (S == 53)
      {
          var saintSylvestre = new Date(uneDate.getFullYear(), 11, 31);
          var saintSylvestreGetDay = saintSylvestre.getDay(); 
          if( anneeBissextile )
          {
              if ( (saintSylvestreGetDay != 4) && (saintSylvestreGetDay != 5) )
              {
                  S = 1;
              }
          }
          else
          {
              if ( (saintSylvestreGetDay != 4) )
              {
                  S = 1;
              }
          }
      }
      
      return S;
  }



  /******************************************************************************/
  /*                                                                            */
  /*                                titrePage                                   */
  /*                                                                            */
  /******************************************************************************/
  /*
   *
   * Renvoie le titre de la page.
   *
   */
   
   function titrePage()
   {
      // Ampute la valeur de <title> de " - Wikipédia"
      //return document.title.replace(" - Wikipédia", "");
      
      // retourne la valeur de <h1 class="firstHeading">
      var page = "";
      var node = document.getElementById("top");
      while(node)
      {
          if (node.className == "firstHeading")
          {
              page = node.innerHTML;
              break;
          }
          node = node.nextSibling;          
      }
      return page;
   }



  /******************************************************************************/
  /*                                                                            */
  /*                               Aujourdhui                                   */
  /*                                                                            */
  /******************************************************************************/
  /*
  *
  *
  *   Paramètre :
  *
  *      format(string) : une valeur parmi {"bistro";"suppression";"oracle"}
  *
  *
  *   Résultat :
  *
  *      cas (format)
  *          "bistro" :
  *              la date du jour sous la forme : "/jj_mmmmmmm_aaaa" (ex : /29_octobre_2005)
  *          "supression" :
  *              la date du jour sous la forme : "#jj_mmmmmmm" (ex : #29_octobre)
  *          "oracle" :
  *              la date du jour sous la forme : "/semaine_ss_aaaa" (ex : /semaine_44_2005)
  */
  
  function Aujourdhui(format)
  {
      var now = new Date();
      var mois = new Array("janvier","février","mars","avril","mai","juin", "juillet","août","septembre","octobre","novembre","décembre");
      
      day = now.getDate();
      month = mois[now.getMonth()];
      year = now.getFullYear();
      
      
      var today = "";
      
      if(format == "bistro")
      {
          today = "/" + day + "_" + month + "_" + year;
      }
      else if(format == "suppression")
      {
          today = "#" + day + "_" + month;
      }
      else if(format == "oracle")
      {
          var week = semaine(now);
          var annee = year;
          if( (week > 51) && (month == "janvier") )
          {
              --annee;
          }
          
          today = "/semaine_" + week + "_" + annee;
      }
      
      return today ;
  }



  /******************************************************************************/
  /*                                                                            */
  /*                        Ajoute une boite personnalisée                      */
  /*                                                                            */
  /******************************************************************************/
  
  function BoitePerso()
  {
      
      var purger = "&action=purge"; // "";
      
      var h5 = document.createElement( "h5" );
      {
          var text = document.createTextNode( "Boîte perso" );
          h5.appendChild( text );
      }
      
      var div_pBody = document.createElement( "div" );
      div_pBody.setAttribute( "class", "pBody" );
      
      {
          var ul_1 = document.createElement( "ul" );
          
          {
              var li_11 = document.createElement( "li" );
              var randompage = document.getElementById( "n-randompage" );
              li_11.appendChild( randompage.cloneNode(true) );
              
              var li_12 = document.createElement( "li" );
              var recentchanges = document.getElementById( "n-recentchanges" );
              li_12.appendChild( recentchanges.cloneNode(true) );
          }
          ul_1.appendChild( li_11 );
          ul_1.appendChild( li_12 );
      }
      div_pBody.appendChild(ul_1);
      
      {
          var hr = document.createElement( "hr" );
      }
      div_pBody.appendChild(hr);
      
      {
          var ul_2 = document.createElement( "ul" );
          
          {
              var li_21 = document.createElement( "li" );
              
              {
                  var a_111 = document.createElement( "a" );
                  a_111.setAttribute( "href", "/w/index.php?title=Wikipédia:Le_Bistro" + Aujourdhui("bistro") + purger);
                  
                  {
                      var text_1111 = document.createTextNode( "Bistro " );
                      a_111.appendChild( text_1111 );
                  }
                  li_21.appendChild( a_111 );
                  
                  var text_112 = document.createTextNode( " | " );
                  li_21.appendChild( text_112 );
                  
                  var a_112 = document.createElement( "a" );
                  a_112.setAttribute( "href", "/w/index.php?title=Wikipédia:Oracle" + Aujourdhui("oracle") + purger);
                  
                  {
                      var text_1121 = document.createTextNode( " Oracle" );
                      a_112.appendChild( text_1121 );
                  }
                  li_21.appendChild( a_112 );
              }
              ul_2.appendChild( li_21 );
              
              
              
              {
                  var li_22 = document.createElement( "li" );
                  
                  {
                      var text_111 = document.createTextNode( "Supprimer " ) ;
                      li_22.appendChild( text_111 );
                      
                      var a_111 = document.createElement( "a" );
                      a_111.setAttribute( "href", "/w/index.php?title=Wikipédia:Pages_à_supprimer" + purger + Aujourdhui("suppression"));
                      
                      {
                          var text_1111 = document.createTextNode( "Pages" );
                          a_111.appendChild( text_1111 );
                      }
                      li_22.appendChild( a_111 );
                      
                      var text_112 = document.createTextNode( " | " );
                      li_22.appendChild( text_112 );
                      
                      var a_112 = document.createElement( "a" );
                      a_112.setAttribute( "href", "/w/index.php?title=Wikipédia:Images_à_supprimer" + purger + Aujourdhui("suppression"));
                      
                      {
                          var text_1121 = document.createTextNode( "Images" );
                          a_112.appendChild( text_1121 );
                      }
                      li_22.appendChild( a_112 );
                  }
                  ul_2.appendChild( li_22 );
              }
          }
          div_pBody.appendChild( ul_2 );
      }
      
      {
          hr = document.createElement( "hr" );
      }
      div_pBody.appendChild(hr);
      
      {
          var ul_3 = document.createElement( "ul" );
          
          {
              var li_31 = document.createElement( "li" );
              
              {
                  var a_311 = document.createElement( "a" );
                  a_311.setAttribute( "href", "/wiki/Wikipédia:Bot/Requêtes");
                  
                  {
                      var text_3111 = document.createTextNode( "Allo Robot ?" );
                      a_311.appendChild( text_3111 );
                  }
                  li_31.appendChild( a_311);
              }
              ul_3.appendChild( li_31 );
              
              
              var li_32 = document.createElement( "li" );
              
              {
                  var text_321 = document.createTextNode( " New " );
                  li_32.appendChild( text_321 );
                  
                  var a_322 = document.createElement( "a" );
                  a_322.setAttribute( "href", "/wiki/Special:Log/newusers");
                  
                  {
                      var text_3221 = document.createTextNode( "Users" );
                      a_322.appendChild( text_3221 );
                  }
                  li_32.appendChild( a_322);
                  
                  var text_323 = document.createTextNode( " | " );
                  li_32.appendChild( text_323 );
                  
                  var a_324 = document.createElement( "a" );
                  a_324.setAttribute( "href", "/wiki/Special:Contributions/newbies");
                  
                  {
                      var text_3241 = document.createTextNode( "Contribs" );
                      a_324.appendChild( text_3241 );
                  }
                  li_32.appendChild( a_324);
              }
              ul_3.appendChild( li_32 );
          }
          
          
          var li_33 = document.createElement( "li" );
          
          {
              var a_332 = document.createElement( "a" );
              a_332.setAttribute( "href", "/wiki/Wikipédia:Éditeurs problématiques");
              
              {
                  var text_3321 = document.createTextNode( "Éd. prob." );
                  a_332.appendChild( text_3321 );
              }
              li_33.appendChild( a_332);
              
              var text_333 = document.createTextNode( " | " );
              li_33.appendChild( text_333 );
              
              var a_334 = document.createElement( "a" );
              a_334.setAttribute( "href", "/wiki/Wikipédia:Vandalisme en cours");
              
              {
                  var text_3341 = document.createTextNode( "Vandalisme (!)" );
                  a_334.appendChild( text_3341 );
              }
              li_33.appendChild( a_334);
          }
          ul_3.appendChild( li_33 );
      }
      
      div_pBody.appendChild( ul_3 );
      
      var div_portlet = document.createElement( "div" ) ;
      div_portlet.setAttribute( "class", "portlet" );
      div_portlet.setAttribute( "id", "p-BoitePerso" ) ;
      
      div_portlet.appendChild( h5 );
      
      div_portlet.appendChild( div_pBody );
      
      div_psearch = document.getElementById( "p-search" );
      document.getElementById("column-one").insertBefore( div_portlet, div_psearch);
      
  }
  
  addLoadEvent(BoitePerso);



  /******************************************************************************/
  /*                                                                            */
  /*                   Ajouter des liens vers les monobooks                     */
  /*                                                                            */
  /******************************************************************************/
  
  /*
   * AjouteLiensMonobooks
   *
   *	Ajoute des liens vers les monobooks (.js et .css) ainsi que vers le
   * 	brouillon perso. Les liens sont placés en haut de la page. (par défaut)
   *
   */
  function AjouteLiensMonobooks()
  {
      var logout = document.getElementById("pt-logout");
      var user = document.getElementById("pt-userpage");
      var titre = titrePage();
      
      var userPage;
      
      if(user)
      {
          var a = user.firstChild;
          if(a)
          {
              userPage = a.getAttribute("href");
          }
      }
      
      var jsA = document.createElement("a");
      jsA.setAttribute("title", "Voir le monobook Javascript");
      var jsAhref = userPage + "/monobook.js";
      jsA.setAttribute("href", jsAhref);
      var jsAText = document.createTextNode("Js");
      jsA.appendChild(jsAText);
            
      var js = document.createElement("li");
      if (titre == jsAhref.replace(/\/wiki\//, ""))
      {
          js.setAttribute("class", "active");
      }
      js.setAttribute("id","pt-monobookJs");
      js.appendChild(jsA);
      
      var cssA = document.createElement("a");
      cssA.setAttribute("title", "Voir le monobook CSS");
      var cssAhref = userPage + "/monobook.css";
      cssA.setAttribute("href", cssAhref);
      var cssAText = document.createTextNode("Css");
      cssA.appendChild(cssAText);
      
      var css = document.createElement("li");
      if (titre == cssAhref.replace(/\/wiki\//, ""))
      {
          css.setAttribute("class", "active");
      }
      css.setAttribute("id","pt-monobookCss");
      css.appendChild(cssA);
      
      var brouillonA = document.createElement("a");
      brouillonA.setAttribute("title", "Brouillon");
      var brouillonAhref = userPage + "/Brouillon";
      brouillonA.setAttribute("href",brouillonAhref);
      var brouillonAText = document.createTextNode("Brouillon");
      brouillonA.appendChild(brouillonAText);
      
      var brouillon = document.createElement("li");
      if (titre == brouillonAhref.replace(/\/wiki\//, ""))
      {
          brouillon.setAttribute("class", "active");
      }
      brouillon.setAttribute("id","pt-brouillon");
      brouillon.appendChild(brouillonA);
    
      var suiviA = document.createElement("a");
      suiviA.setAttribute("title", "Liste de suivi ++");
      var suiviAhref = userPage + "/suivi";
      suiviA.setAttribute("href",suiviAhref);
      var suiviAText = document.createTextNode("Suivi");
      suiviA.appendChild(suiviAText);
      
      var suivi = document.createElement("li");
      if (titre == suiviAhref.replace(/\/wiki\//, ""))
      {
          suivi.setAttribute("class", "active");
      }
      suivi.setAttribute("id","pt-suivi");
      suivi.appendChild(suiviA);
      
      if(logout)
      {
          logout.parentNode.insertBefore(brouillon, logout);
          logout.parentNode.insertBefore(css, logout);
          logout.parentNode.insertBefore(js, logout);
          logout.parentNode.insertBefore(suivi, logout);
      }
  }
  addLoadEvent(AjouteLiensMonobooks);



  /******************************************************************************/
  /*                                                                            */
  /*                     Catégories dans le menu latéral                        */
  /*                                                                            */
  /******************************************************************************/
  
  /*
   * CategoriesAGauche
   *
   *	  Recopie les catégories dans la colonne de gauche (par défaut).
   *
   */
  
  function CategoriesAGauche()
  {	
      var catlinks = document.getElementById("catlinks");
      if (!catlinks)
      {
          return;
      }
      var categories = catlinks.getElementsByTagName("a") ;
      
      
      var h5_cats = document.createElement( "h5" );
      var text = document.createTextNode( "Catégories" ) ;
      h5_cats.appendChild( text ) ;
      
      var div_cats = document.createElement( "div" );
      div_cats.setAttribute( "class", "pBody" );
      
      var ul = document.createElement( "ul" );
      
      var a_debut = 1 ;
      for (var i = a_debut ; i < categories.length ; i++ )
      {
          var a = categories[i] ;
          var li = document.createElement( "li" ) ;
          li.appendChild( a.cloneNode(true) ) ;
          ul.appendChild( li );
      }
      
      
      div_cats.appendChild( ul ) ;
      
      
      var portlet = document.createElement( "div" ) ;
      portlet.setAttribute( "class", "portlet" );
      portlet.setAttribute( "id", "p-cat" ) ;
      
      portlet.appendChild( h5_cats ) ;
      
      portlet.appendChild( div_cats ) ;
      
      div_ptb = document.getElementById( "p-tb" );
      document.getElementById("column-one").insertBefore( portlet, div_ptb)
      
  }
  
  addLoadEvent(CategoriesAGauche);



  /******************************************************************************/
  /*                                                                            */
  /*                               Onglets Google                               */
  /*                                                                            */
  /******************************************************************************/
  
  /*
   * OngletGoogle
   *
   *	  Ajoute un onglet Google, pour des recherches dans wikiedia et sur le web.
   *
   */
  
  function OngletGoogle()
  {
      var a = document.getElementById("p-cactions");
      if (a) 
      {
          var titre = titrePage();
          var googleURL = "http://www.google.fr/search?num=20&hl=en&safe=off&q=";
          b = a.getElementsByTagName("ul");
      
          if(b.length > 0)
          {
              b[0].innerHTML = b[0].innerHTML
              + '<li id="ca-nstab-user">'
              + '<img src="http://www.google.com/nav_first.gif" height="13" width="9"/>'
              + '<a href="' + googleURL
              + 'site%3Afr.wikipedia.org+%22' + titre + '%22&btnG=Search">wp-fr</a>'
              + '<a href="' + googleURL
              + 'site%3Awikipedia.org+%22' + titre + '%22&btnG=Search">wp</a>'
              + '<a href="' + googleURL
              + '%22' + titre + '%22&btnG=Search&meta=lr%3Dlang_fr">web-fr</a>'
              + '<a href="' + googleURL
              + '%22' + titre + '%22&btnG=Search">web</a></li>';
          }
      }
  }
  
  
  addLoadEvent(OngletGoogle);



  /******************************************************************************/
  /*                                                                            */
  /*                            Onglets en bas de page                          */
  /*                                                                            */
  /******************************************************************************/
  
  /*
  * AjouteOnglets
  *
  *		clone les onglets en bas de page.
  *
  */
  function AjouteOnglets()
  {
      var tabs = document.getElementById('p-cactions').cloneNode(true);
      tabs.id = 'mytabs';
      var li = tabs.getElementsByTagName('li');
      for (i = 0; i < li.length; i++)
      {
          if (li[i].id)
          {
              li[i].id = 'mytabs-' + li[i].id;
          }
      }
      document.getElementById('column-content').appendChild(tabs);
  }
  
  addLoadEvent(AjouteOnglets);
  
  
    /******************************************************************************/
  /*                                                                            */
  /*                  Boutons supplémentaires en mode édition                   */
  /*                                                                            */
  /******************************************************************************/
  
  function MiluxeBar()
  {
	  if (document.createTextNode)
	  {
		  var toolbar = document.getElementById("toolbar"); 
		  if (!toolbar) return;

                  var uploadFr = "http://upload.wikimedia.org/wikipedia/fr";
		  
		  ///// zone personnalisable ////
		  var messages = new Array();
		  var images = new Array();
		  var commentaires = new Array();
		  var overwrite = new Array();
                  var display = new Array();
                  var titres = new Array();

                  var page = titrePage().replace(/modification de /, "");		  
		  
		  // le texte a été splitté sinon Mediawiki casse tout...
                  messages[0] = "{{subst:" + "Bienvenu"  + "}}" +  "~~" + "~~";
                  images[0] = uploadFr + "/1/12/Button_accueilA.png";
                  commentaires[0] = "Bienvenue";
                  overwrite[0] = true;
                  display[0] = (page.indexOf("Utilisateur") == 0); // pages Utilisateur
                  titres[0] = "Bienvenue Utilisateur !";
		  
		  messages[1] = "{{subst:" + "Bienvenue nouveau" + "}}" +  "~~" + "~~";
                  images[1] = uploadFr + "/e/eb/Button_accueilB.png";
                  commentaires[1] = "Bienvenue";
                  overwrite[1] = true;
                  display[1] = (page.indexOf("Discussion Utilisateur") == 0); // pages de discussion Utilisateur
                  titres[1] = "Bienvenue Page de discussion !";
                  
		  messages[2] = "{{subst:" + "Bienvenue vandale" + "}}" +  "~~" + "~~";
                  images[2] = uploadFr + "/5/5f/Button_police.png";
                  commentaires[2] = "Avertissement vandalisme";
                  overwrite[2] = true;
                  display[2] = display[1]; // pages de discussion Utilisateur
                  titres[2] = "Avertissement vandalisme !";
                  
		  messages[3] = "{{subst:" + "Vandalisme|page" + "}}" +  "~~" + "~~";
                  images[3] = uploadFr + "/7/79/Button_bombe.png";		  
		  commentaires[3] = "Avertissement vandalisme";
		  overwrite[3] = true;
                  display[3] = display[1]; // pages de discussion Utilisateur
                  titres[3] = "Avertissement vandalisme sur la page ...";
		  
		  
		  messages[4] = "{{subst:" + "Bienvenue spammeur" + "}}" +  "~~" + "~~";
		  images[4] = uploadFr + "/f/fb/Button_spam2.png";
		  commentaires[4] = "Avertissement spam";
		  overwrite[4] = true;
                  display[4] = display[1]; // pages de discussion Utilisateur
                  titres[4] = "Avertissement spam !";
		  
		  messages[5] = "{{subst:" + "Bienvenue débutant" + "}}" +  "~~" + "~~";
		  images[5] = uploadFr + "/a/ad/Button_vandale2.png";
		  commentaires[5] = "Bienvenue";
		  overwrite[5] = true;
                  display[5] = display[1]; // pages de discussion Utilisateur
                  titres[5] = "Bienvenue débutant !";
		  
		  messages[6] = "{{" + "suppression" + "}}";
		  images[6] = uploadFr + "/f/f8/Button_suppr.png";
		  commentaires[6] = "Proposé à la suppression";
		  overwrite[6] = true;
                  display[6] = ! (display[0] || display[1]) ; // tout sauf pages de discussion Utilisateur et pages Utilisateur
                  titres[6] = "Proposé à la suppression";
		  
		  messages[7] = "* {{conserver" + "}} " +  "~~" + "~~";
		  images[7] = uploadFr + "/6/60/Button_support.png";
		  commentaires[7] = "/* Vote */ conserver";
		  overwrite[7] = true;
                  display[7] = display[6];  // tout sauf pages de discussion Utilisateur et pages Utilisateur
                  titres[7] = "Voter CONSERVER";
		  
		  messages[8] = "* {{supprimer" + "}} " +  "~~" + "~~";
		  images[8] = uploadFr + "/9/98/Button_oppose.png";
		  commentaires[8] = "/* Vote */ supprimer";
		  overwrite[8] = true;
                  display[8] = display[6];  // tout sauf pages de discussion Utilisateur et pages Utilisateur
                  titres[8] = "Voter SUPPRIMER";
		  
		  messages[9] = "{{subst:" + "Utilisateur:Eskimo/Copyvio|page|URL|titre" + "}} " +  "~~" + "~~";
		  images[9] = uploadFr + "/d/d3/Button_copyvio_discuss.png";
		  commentaires[9] = "Violation de copyright";
		  overwrite[9] = true;
                  display[9] = display[6];  // tout sauf pages de discussion Utilisateur et pages Utilisateur
                  titres[9] = "Violation de copyright : page, URL, titre";

		  messages[10] = "{{" + "copie de site|URL|Titre" + "}}";
		  images[10] = uploadFr + "/e/e5/Button_copyvio.png";
		  commentaires[10] = "Violation de copyright";
		  overwrite[10] = true;
                  display[10] = display[6];  // tout sauf pages de discussion Utilisateur et pages Utilisateur
                  titres[10] = "Violation de copyright";
		  
		  messages[11] = "{{" + "pdf" + "}} ";
		  images[11] = "http://www.itb.cnr.it/flex/TemplatesUSR/CM/TemplatesUSR-BLOB-img/Download/ACROBAT-PDF-small.gif";
		  commentaires[11] = " + indication de format : PDF";
		  overwrite[11] = false;
                  display[11] = display[6];  // tout sauf pages de discussion Utilisateur et pages Utilisateur
                  titres[11] = "indication de format PDF";

                  messages[12] = "";
		  images[12] = uploadFr + "/e/e3/Button_disambiguation.png";
		  commentaires[12] = " Correction des [[Wikipédia:Projet/Liens vers les pages d'homonymie|liens vers les pages d'homonymie]]";
		  overwrite[12] = false;
                  display[12] = true;
                  titres[12] = "Homonymie";

                  messages[13] = "";
		  images[13] = uploadFr + "/8/84/Button_vide.png";
		  commentaires[13] = "";
		  overwrite[13] = false;
                  display[13] = true;
                  titres[13] = "Blanchir la page";
                    
                  ////////////////////////////////
		  
		  for (var i=0;i<messages.length;i++)
		  {
                      if (display[i])
                      {
			  var img = document.createElement("img");
			  img.setAttribute("src", images[i]);
                          img.setAttribute("title", titres[i]);
			  
                          var ref = document.createElement("a");
                          if(i != 13)
                          {
                              var ahref = "javascript:insertTags('', \"";
                              ahref += messages[i];
                              ahref += "\", ''); javascript:changeSummary(\"";
                              ahref += commentaires[i];
                              ahref += "\", " + overwrite[i] + ");";
                          }
                          else // i == 13
                          {
                              var ahref = "javascript:blanchir();";                             
                          }                              
                          ref.setAttribute("href", ahref); 
                          ref.appendChild(img);
                          toolbar.appendChild(ref);
                       }
		  }
	  }
  }
  
  function changeSummary(sampleText, overwrite)
  {
	  if (overwrite)
	  {
		  document.editform.wpSummary.value = sampleText;
	  }
          else if (document.editform.wpSummary.value.indexOf(sampleText) == -1)
	  {
		  document.editform.wpSummary.value += sampleText;
	  }
	  
  }

  function blanchir()
  {
      var text = document.editform.wpTextbox1.value.substring(0, 176);
      while(text.search(/\n/) != -1)
      {
          text = text.replace(/\n/," ");
      }
      var resume = "blanchi : contenait « " + text + " »";
      changeSummary(resume, false);
      
      document.editform.wpTextbox1.value = "";
  }
  
  addLoadEvent(MiluxeBar);
  




  // MERCI Sam !!
  // La source de ce magnifique script est : "http://sam.zoy.org/wikipedia/godmode-light.js"
  //
  // j'y ai fait quelques petites modifs :o)
  
  // -----------------------------------------------------------------------------
  // God-like Monobook skin
  // (c) 2005 Sam Hocevar <sam@zoy.org>
  // $Id: godmode-light.js 980 2005-11-12 01:51:51Z sam $
  // -----------------------------------------------------------------------------
  
  // -----------------------------------------------------------------------------
  // Language support, taken from phase3/languages/*
  // -----------------------------------------------------------------------------
  var rollbacklink = 'rollback';
  var cantrollback = 'Cannot revert edit; last contributor is only author of this page.';
  var alreadyrolled = 'Cannot rollback last edit of [[$1]] by [[User:$2|$2]] ([[User talk:$2|Talk]]); someone else has edited or rolled back the page already. Last edit was by [[User:$3|$3]] ([[User talk:$3|Talk]]). ';
  var revertpage = 'Reverted edits by [[Special:Contributions/$2|$2]] to last version by [[User:$1|$1]]';
  switch (document.getElementsByTagName('html')[0].lang) {
      case 'fr':
      rollbacklink = 'révoquer';
      cantrollback = 'Impossible de révoquer: dernier auteur est le seul à avoir modifié cet article';
      alreadyrolled = 'Impossible de révoquer la dernière modification de [[$1]] par  [[User:$2|$2]] ([[User talk:$2|Talk]]); quelqu\'un d\'autre a déjà modifié ou révoqué l\'article. La dernière modification était de [[User:$3|$3]] ([[User talk:$3|Talk]]). '; // lol @ pathetic grammar
      revertpage = "Révocation des modifications de [[Special:Contributions/$2|$2]] ([[Discussion Utilisateur:$2|Discuter]]) et restauration d'une précédente version de [[User:$1|$1]]";
      break;
      case 'de':
      rollbacklink = 'Zurücksetzen';
      cantrollback = 'Die Änderung kann nicht zurückgenommen werden; der letzte Autor ist der einzige.';
      alreadyrolled = 'Die Zurücknahme des Artikels [[$1]] von [[Benutzer:$2|$2]] ([[Benutzer Diskussion:$2|Diskussion]]) ist nicht möglich, da eine andere Änderung oder Rücknahme erfolgt ist.  Die letzte Änderung ist von [[Benutzer:$3|$3]] ([[Benutzer Diskussion:$3|Diskussion]])';
      revertpage = 'Änderungen von [[Benutzer:$2]] rückgängig gemacht und letzte Version von [[Benutzer:$1]] wiederhergestellt';
      break;
      case 'es':
      rollbacklink = 'Revertir';
      cantrollback = 'No se pueden revertir las ediciones; el último colaborador es el único autor de este artículo.';
      alreadyrolled = 'No se puede revertir la última edición de [[$1]] por [[Colaborador:$2|$2]] ([[Colaborador Discusión:$2|Discusión]]); alguien más ya ha editado o revertido esa página.  La última edición fue hecha por [[Colaborador:$3|$3]] ([[Colaborador Discusión:$3|Discusión]]). ';
      revertpage = 'Revertida a la última edición de $1';
      break;
      case 'it':
      rollbacklink = 'rollback';
      cantrollback = 'Impossibile tornare ad una versione precedente: l\'ultima modifica è stata apportata dall\'unico utente che abbia lavorato a questo articolo.';
      //alreadyrolled = '';
      revertpage = 'Riportata alla revisione precedente da $1';
      break;
      case 'pt':
      rollbacklink = 'voltar';
      cantrollback = 'Não foi possível reverter a edição; o último contribuidor é o único autor deste artigo.';
      alreadyrolled = 'Não foi possível reverter as edições de  [[$1]] por [[User:$2|$2]] ([[User talk:$2|Talk]]); alguém o editou ou já o reverteu.  A última edição foi de  [[User:$3|$3]] ([[User talk:$3|Conversar com ele]]). ';
      revertpage = 'Revertidas edições por [[Special:Contributions/$2|$2]], para a última versão por $1';
      break;
  }
  
  // -----------------------------------------------------------------------------
  // XMLHttpRequest support
  // -----------------------------------------------------------------------------
  if (document.implementation.createDocument)
  {
      var xmlparser = new DOMParser();
  }
  
  function XMLParse(string)
  {
      if (document.implementation.createDocument)
      {
          return xmlparser.parseFromString(string, "text/xml");
      } else if (window.ActiveXObject)
      {
          var xmldoc = new ActiveXObject("Microsoft.XMLDOM");
          xmldoc.async = "false";
          ret = xmldoc.loadXML(string);      
          if (!ret)
              return null;
          return xmldoc.documentElement;
      }
      return null;
  }
  
  var xmlhttp;
  
  function HTTPClient()
  {
      var http;
      if(window.XMLHttpRequest)
      {
          http = new XMLHttpRequest();
      } else if (window.ActiveXObject)
      {
          try
          {
              http = new ActiveXObject("Msxml2.XMLHTTP");
          }
          catch (e)
          {
              try
              {
                  http = new ActiveXObject("Microsoft.XMLHTTP");
              } catch (E)
              {
                  http = false;
              }
          }
      }
      return http;
  }
  
  // -----------------------------------------------------------------------------
  // MD5 hash calculator
  // -----------------------------------------------------------------------------
  // Version 2.1 Copyright (C) Paul Johnston 1999 - 2002.
  // Other contributors: Greg Holt, Andrew Kepert, Ydnar, Lostinet
  // Distributed under the BSD License
  // See http://pajhome.org.uk/crypt/md5 for more info.
  // -----------------------------------------------------------------------------
  var hexcase = 0;
  var b64pad  = "";
  var chrsz   = 8;
  
  function hex_md5(s)
  {
      return binl2hex(core_md5(str2binl(s), s.length * chrsz));
  }
  
  function core_md5(x, len)
  {
      x[len >> 5] |= 0x80 << ((len) % 32);
      x[(((len + 64) >>> 9) << 4) + 14] = len;
      
      var a =  1732584193;
      var b = -271733879;
      var c = -1732584194;
      var d =  271733878;
      
      for(var i = 0; i < x.length; i += 16)
      {
          var olda = a;
          var oldb = b;
          var oldc = c;
          var oldd = d;
          
          a = md5_ff(a, b, c, d, x[i+ 0], 7 , -680876936); d = md5_ff(d, a, b, c, x[i+ 1], 12, -389564586); c = md5_ff(c, d, a, b, x[i+ 2], 17,  606105819); b = md5_ff(b, c, d, a, x[i+ 3], 22, -1044525330); a = md5_ff(a, b, c, d, x[i+ 4], 7 , -176418897); d = md5_ff(d, a, b, c, x[i+ 5], 12,  1200080426); c = md5_ff(c, d, a, b, x[i+ 6], 17, -1473231341); b = md5_ff(b, c, d, a, x[i+ 7], 22, -45705983); a = md5_ff(a, b, c, d, x[i+ 8], 7 ,  1770035416); d = md5_ff(d, a, b, c, x[i+ 9], 12, -1958414417); c = md5_ff(c, d, a, b, x[i+10], 17, -42063); b = md5_ff(b, c, d, a, x[i+11], 22, -1990404162); a = md5_ff(a, b, c, d, x[i+12], 7 ,  1804603682); d = md5_ff(d, a, b, c, x[i+13], 12, -40341101); c = md5_ff(c, d, a, b, x[i+14], 17, -1502002290); b = md5_ff(b, c, d, a, x[i+15], 22,  1236535329);
          
          a = md5_gg(a, b, c, d, x[i+ 1], 5 , -165796510); d = md5_gg(d, a, b, c, x[i+ 6], 9 , -1069501632); c = md5_gg(c, d, a, b, x[i+11], 14,  643717713); b = md5_gg(b, c, d, a, x[i+ 0], 20, -373897302); a = md5_gg(a, b, c, d, x[i+ 5], 5 , -701558691); d = md5_gg(d, a, b, c, x[i+10], 9 ,  38016083); c = md5_gg(c, d, a, b, x[i+15], 14, -660478335); b = md5_gg(b, c, d, a, x[i+ 4], 20, -405537848); a = md5_gg(a, b, c, d, x[i+ 9], 5 ,  568446438); d = md5_gg(d, a, b, c, x[i+14], 9 , -1019803690); c = md5_gg(c, d, a, b, x[i+ 3], 14, -187363961); b = md5_gg(b, c, d, a, x[i+ 8], 20,  1163531501); a = md5_gg(a, b, c, d, x[i+13], 5 , -1444681467); d = md5_gg(d, a, b, c, x[i+ 2], 9 , -51403784); c = md5_gg(c, d, a, b, x[i+ 7], 14,  1735328473); b = md5_gg(b, c, d, a, x[i+12], 20, -1926607734);
          
          a = md5_hh(a, b, c, d, x[i+ 5], 4 , -378558); d = md5_hh(d, a, b, c, x[i+ 8], 11, -2022574463); c = md5_hh(c, d, a, b, x[i+11], 16,  1839030562); b = md5_hh(b, c, d, a, x[i+14], 23, -35309556); a = md5_hh(a, b, c, d, x[i+ 1], 4 , -1530992060); d = md5_hh(d, a, b, c, x[i+ 4], 11,  1272893353); c = md5_hh(c, d, a, b, x[i+ 7], 16, -155497632); b = md5_hh(b, c, d, a, x[i+10], 23, -1094730640); a = md5_hh(a, b, c, d, x[i+13], 4 ,  681279174); d = md5_hh(d, a, b, c, x[i+ 0], 11, -358537222); c = md5_hh(c, d, a, b, x[i+ 3], 16, -722521979); b = md5_hh(b, c, d, a, x[i+ 6], 23,  76029189); a = md5_hh(a, b, c, d, x[i+ 9], 4 , -640364487); d = md5_hh(d, a, b, c, x[i+12], 11, -421815835); c = md5_hh(c, d, a, b, x[i+15], 16,  530742520); b = md5_hh(b, c, d, a, x[i+ 2], 23, -995338651);
          
          a = md5_ii(a, b, c, d, x[i+ 0], 6 , -198630844); d = md5_ii(d, a, b, c, x[i+ 7], 10,  1126891415); c = md5_ii(c, d, a, b, x[i+14], 15, -1416354905); b = md5_ii(b, c, d, a, x[i+ 5], 21, -57434055); a = md5_ii(a, b, c, d, x[i+12], 6 ,  1700485571); d = md5_ii(d, a, b, c, x[i+ 3], 10, -1894986606); c = md5_ii(c, d, a, b, x[i+10], 15, -1051523); b = md5_ii(b, c, d, a, x[i+ 1], 21, -2054922799); a = md5_ii(a, b, c, d, x[i+ 8], 6 ,  1873313359); d = md5_ii(d, a, b, c, x[i+15], 10, -30611744); c = md5_ii(c, d, a, b, x[i+ 6], 15, -1560198380); b = md5_ii(b, c, d, a, x[i+13], 21,  1309151649); a = md5_ii(a, b, c, d, x[i+ 4], 6 , -145523070); d = md5_ii(d, a, b, c, x[i+11], 10, -1120210379); c = md5_ii(c, d, a, b, x[i+ 2], 15,  718787259); b = md5_ii(b, c, d, a, x[i+ 9], 21, -343485551);
          
          a = safe_add(a, olda);
          b = safe_add(b, oldb);
          c = safe_add(c, oldc);
          d = safe_add(d, oldd);
      }
      return Array(a, b, c, d);
      
  }
  
  function md5_cmn(q, a, b, x, s, t)
  {
      return safe_add(bit_rol(safe_add(safe_add(a, q), safe_add(x, t)), s),b);
  }
  function md5_ff(a, b, c, d, x, s, t)
  {
      return md5_cmn((b & c) | ((~b) & d), a, b, x, s, t);
  }
  function md5_gg(a, b, c, d, x, s, t)
  {
      return md5_cmn((b & d) | (c & (~d)), a, b, x, s, t);
  }
  function md5_hh(a, b, c, d, x, s, t)
  {
      return md5_cmn(b ^ c ^ d, a, b, x, s, t);
  }
  function md5_ii(a, b, c, d, x, s, t)
  {
      return md5_cmn(c ^ (b | (~d)), a, b, x, s, t);
  }
  
  function safe_add(x, y)
  {
      var lsw = (x & 0xFFFF) + (y & 0xFFFF);
      var msw = (x >> 16) + (y >> 16) + (lsw >> 16);
      return (msw << 16) | (lsw & 0xFFFF);
  }
  
  function bit_rol(num, cnt)
  {
      return (num << cnt) | (num >>> (32 - cnt));
  }
  
  function str2binl(str)
  {
      var bin = Array();
      var mask = (1 << chrsz) - 1;
      for(var i = 0; i < str.length * chrsz; i += chrsz)
          bin[i>>5] |= (str.charCodeAt(i / chrsz) & mask) << (i%32);
      return bin;
  }
  
  function binl2hex(binarray)
  {
      var hex_tab = hexcase ? "0123456789ABCDEF" : "0123456789abcdef";
      var str = "";
      for(var i = 0; i < binarray.length * 4; i++)
      {
          str += hex_tab.charAt((binarray[i>>2] >> ((i%4)*8+4)) & 0xF) +
          hex_tab.charAt((binarray[i>>2] >> ((i%4)*8  )) & 0xF);
      }
      return str;
  }
  
  // -----------------------------------------------------------------------------
  // Our nice Revert functions
  // -----------------------------------------------------------------------------
  var gml_vandal, gml_editor, gml_url;
  
  function PerformRevert()
  {
      var l, token = '', revert = false;
      // Look for '&fakeaction=rollback' in URL
      gml_url = location.pathname;
      l = location.search.substring(1).split('&');
      for (i = 0; i < l.length; i++)
      {
          var n = l[i].indexOf('=');
          var name = l[i].substring(0, n);
          if (name == 'fakeaction')
          {
              if (l[i].substring(n + 1) == 'rollback')
                  revert = true;
          }
          else if (name == 'vandal')
          {
              gml_vandal = unescape(l[i].substring(n + 1));
          }
          else if (name == 'token')
          {
              token = unescape(l[i].substring(n + 1));
          }
          else if (name == 'title')
          {
              gml_url += '?' + l[i];
          }
      }
      if (!revert)
          return;
      document.getElementById('bodyContent').innerHTML = 'Please wait, reverting edits by ' + gml_vandal + '...';
      // Avoid XSS kiddies by using a special token
      if (token == '' || token != hex_md5(gml_url + gml_vandal + document.cookie))
      {
          document.getElementById('bodyContent').innerHTML += '<br />Bad authentication token!';
          return;
      }
      
      xmlhttp = HTTPClient();
      if (!xmlhttp)
          return;
      document.getElementById('bodyContent').innerHTML += '<br />Getting article history...';
      xmlhttp.open("GET", gml_url + '&action=history&limit=50', true);
      xmlhttp.onreadystatechange = RevertStepTwo;
      xmlhttp.send(null);
  }
  
  function RevertStepTwo()
  {
      if (xmlhttp.readyState != 4)
          return
      var l;
      var oldid;
      // Get the vandal and new editor names
      gml_vandal = gml_vandal.replace(/_/g, ' ');
      gml_editor = '';
      doc = XMLParse(xmlhttp.responseText);
      l = doc.getElementById('pagehistory').getElementsByTagName('li');
      //l = doc.selectSingleNode('//*[@id="pagehistory"]').getElementsByTagName('li');
      for (i = 0; i < l.length; i++)
      {
          var name = l[i].getElementsByTagName('span')[0].getElementsByTagName('a')[0].innerHTML.replace(/_/g, ' ');
          if (i == 0 && name != gml_vandal)
          {
              document.getElementById('bodyContent').innerHTML += '<br />Error: Last editor is ' + name + ', not ' + gml_vandal + '!';
              return;
          } else if (i > 0 && name != gml_vandal)
          {
              oldid = l[i].getElementsByTagName('input')[0].value;
              gml_editor = name;
              break;
          }
      }
      if (gml_editor == '')
      {
          document.getElementById('bodyContent').innerHTML += '<br />Error: ' + gml_vandal + ' is the only editor!';
          return;
      }
      
      xmlhttp = HTTPClient();
      if (!xmlhttp)
          return;
      document.getElementById('bodyContent').innerHTML += '<br />Getting article edit form (GET' + gml_url + '&action=edit&oldid=' + oldid + ')...';
      xmlhttp.open('GET', gml_url + '&action=edit&oldid=' + oldid, true);
      xmlhttp.onreadystatechange = RevertStepThree;
      xmlhttp.send(null);
  }
  
  function RevertStepThree()
  {
      if (xmlhttp.readyState != 4)
          return
      var form, newform, l;
      // Insert the downloaded form in our current page, using
      // only hidden form inputs.
      doc = XMLParse(xmlhttp.responseText);
      form = doc.getElementById('editform');
      newform = document.createElement('form');
      l = form.getElementsByTagName('textarea');
      for (i = l.length; i--; )
      {
          var t = document.createElement('input');
          t.type = 'hidden';
          t.name = l[i].name;
          t.value = l[i].value;
          newform.appendChild(t);
      }
      l = form.getElementsByTagName('input');
      for (i = l.length; i--; )
      {
          if (l[i].name == 'wpSummary')
          {
              l[i].value = revertpage.replace(/\$1/g, gml_editor).replace(/\$2/g, gml_vandal);
          }
          else if (l[i].name == 'wpMinoredit')
          {
              l[i].value = '1';
          }
          else if (l[i].name == 'wpWatchthis')
          {
              if (!l[i].checked)
                  continue; // Don’t touch the "watch" status
              l[i].value = "on";
          }
          else if (l[i].name == 'wpPreview')
          {
              continue;
          }
          else if (l[i].name == 'wpDiff')
          {
              continue;
          }
          l[i].type = 'hidden';
          newform.appendChild(l[i]);
      }
      newform.name = form.name;
      newform.method = form.method;
      newform.id = form.id;
      newform.action = form.action;
      document.getElementById('bodyContent').innerHTML += '<br />Submitting form...';
      document.getElementById('bodyContent').appendChild(newform);
      // Submit the form
      newform.submit();
  }
  
  // -----------------------------------------------------------------------------
  // Add revert buttons to the page
  // -----------------------------------------------------------------------------
  function AddRevertButtons()
  {
      var l, article = '', vandal;
      // Add 'revert' links to a diff page
      l = document.getElementById('bodyContent').getElementsByTagName('td');
      for (i = 0; i < l.length; i++)
      {
          if (l[i].className == 'diff-otitle')
          {
              article = l[i].getElementsByTagName('a')[0].href.split('&')[0].replace(/[^\/]*\/\/[^\/]*/, '');
          }
          else if (l[i].className == 'diff-ntitle')
          {
              var toplink = l[i].getElementsByTagName('a')[0].href;
              vandal = l[i].getElementsByTagName('a')[1].title.split(':')[1];
              var t = l[i].innerHTML;
              n = t.indexOf('</a>) <br') + t.indexOf('</A>) <BR') + 1; // XXX: WOW HACK!!!!
              if (n >= 0 && article != '' && toplink.indexOf('oldid=') == -1)
              {
                  l[i].innerHTML = t.substring(0, n + 5) + ' &nbsp;&nbsp;&nbsp;<strong>[<a href="' + article + '&fakeaction=rollback&vandal=' + vandal + '&token=' + hex_md5(article + vandal + document.cookie) + '">' + rollbacklink + '</a>]</strong> ' + t.substring(n + 5, t.length);
              }
          }
      }
      
      // Add 'revert' links to a contributions page
      if (location.href.indexOf(':Contributions') != -1)
      {
          var c = document.getElementById('contentSub');
          var a = c.getElementsByTagName('a');
          if (a.length == 2)
          {
              vandal = a[0].innerHTML;
          }
          else
          {
              vandal = c.innerHTML.replace(/ \(.*/, '').replace(/.* /, '');
          }
          l = document.getElementById('bodyContent').getElementsByTagName('li');
          for (i = 0; i < l.length; i++)
          {
              var t = l[i].innerHTML;
              // If we are already a sysop on this wiki, abort
              if (t.indexOf('>' + rollbacklink + '</a>]') != -1)
              break;
              //if (t.indexOf('&amp;diff=0') != -1)
              {
                  if (t.indexOf('<strong> (') != -1)
                  {
                      article = l[i].getElementsByTagName('a')[0].href.split('&')[0].replace(/[^\/]*\/\/[^\/]*/, '');
                      l[i].innerHTML += ' [<a href="' + article + '&fakeaction=rollback&vandal=' + vandal + '&token=' + hex_md5(article + vandal + document.cookie) + '">' + rollbacklink + '</a>]';
                  }
              }
          }
      }
  }
  
  
  addLoadEvent(PerformRevert);
  addLoadEvent(AddRevertButtons);